'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _output = require('./output');

var _output2 = _interopRequireDefault(_output);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _http = require('./http');

var _http2 = _interopRequireDefault(_http);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Command extends _output2.default {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.http = new _http2.default(this), this.flags = {}, _temp;
  }

  // static _flags: Flags = [
  //   {name: 'debug', hidden: true},
  //   {name: 'no-color', hidden: true}
  // ]

  static get id() {
    return this.command ? `${this.topic}:${this.command}` : this.topic;
  }

  /**
   * instantiate and run the command setting {mock: true} in the config
   */
  static async mock(argv = [], options = {}, ...rest) {
    options.mock = true;
    return this.run(argv, options);
  }

  /**
   * instantiate and run the command
   */
  static async run(argv = [], options = {}, ...rest) {
    let config = new _config2.default(options);
    let cmd = new this(config);
    cmd.argv = argv;
    // if (this.flags.debug) this.config.debug = 1
    try {
      const args = await cmd.parse();
      await cmd.run(args);
      await cmd.done();
    } catch (err) {
      if (config.mock) throw err;
      cmd.error(err);
    }
    return cmd;
  }

  async parse() {
    const parser = new _parser2.default({
      flags: this.constructor.flags || {},
      args: this.constructor.args || [],
      variableArgs: this.constructor.variableArgs,
      cmd: this
    });
    const { argv, flags } = await parser.parse({ flags: this.flags, argv: this.argv });
    this.flags = flags;
    this.argv = argv;
  }

  // prevent setting things that need to be static


  /**
   * actual command run code goes here
   */
  async run(...rest) {}
}
exports.default = Command; // eslint-disable-line

Command.aliases = [];
Command.variableArgs = false;
Command.args = [];